# SPDX-FileCopyrightText: © 2025 DSLab - Fondazione Bruno Kessler
#
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from enum import Enum


class LocalClientVar(Enum):
    """
    Variables for Local.
    """

    EMPTY = "EMPTY"
